### R code from vignette source 'aRTgpm.Rnw'

###################################################
### code chunk number 1: "Rsettings"
###################################################
options(width=69)


###################################################
### code chunk number 2: "Loading package"
###################################################
require(aRT) 
con=openConn(name="default")


###################################################
### code chunk number 3: "Importing data"
###################################################
db=createDb(con, "gpm", replace=TRUE)

llotes = createLayer(db, "lotes")
lcomunidades = createLayer(db, "comunidades")
lrodovias = createLayer(db, "rodovias")

comufn = system.file("shapes/comunidades_UTM.shp", package="aRT")
rodofn = system.file("shapes/rodovias_UTM.shp", package="aRT")
lotefn = system.file("shapes/lotes_UTM.shp", package="aRT")

addShape(lcomunidades, tab="comus", file=comufn, ID="objeid_37")
addShape(lrodovias, tab="rodos", file=rodofn,       ID="objeid_57")
addShape(llotes, tab="lotes", file=lotefn,       ID="id")



rm(db)
invisible(gc())


###################################################
### code chunk number 4: "Opening database"
###################################################
db=openDb(con, "gpm")

llotes = openLayer(db, "lotes")
lcomunidades = openLayer(db, "comunidades")
lrodovias = openLayer(db, "rodovias")

rodovias    = getLines(lrodovias)
comunidades = getPoints(lcomunidades)
lotes       = getPolygons(llotes)


###################################################
### code chunk number 6: "Importing data"
###################################################
lcells = createLayer(llotes, "cells", 150) ## ORIGINAL IS 150
cells  = getCells(lcells, slice=3000) # le de 3000 em 3000, fica bem mais rapido
centroids = getOperation(llotes, operation="centroid")

###################################################
### code chunk number 17: "GPM open network"
###################################################
data = getData(openTable(lrodovias))
network = createOpenNetwork(comunidades, rodovias, function(d, id) {
  pos = which(data[,"OBJEID_57"] == id)
  if(length(pos) == 1 && data[pos, "CD_PAVIMEN"] == "pavimentada")
  {
	print("d/5")
	return(d/5)
  }
  else
  {
	print("d/2")
	return(d/2)
  }
})


get_neighbors_net  = getNeighborsOpenNetworkFunction(centroids, network)
gpmnetwork   = createGPM(llotes, get_neighbors_net)


###################################################
### code chunk number 18: aRTgpm.Rnw:320-354
###################################################
plot_neighborhood_points = function(start_points, end_points, end_points_colors, neighborhood, lines, border)
{
    id_end_points       = getID(end_points)
    length_end_points   = length(id_end_points)
    length_start_points = length(getID(start_points))

    plot(end_points)

    for(i in 1:length_start_points)
    {
        mywhich = which(neighborhood[[i]]$ids == id_end_points)
        if(length(mywhich) > 0)
        {
            col = end_points_colors[mywhich]
            plot(lotes[i,], col=col, pch=20, add=T)
        }
    }

    plot(lines, lwd=2, add=T)

    for(i in 1:length_end_points)
    {
        col = end_points_colors[i]
        id = getID(end_points)[i]
        plot(end_points[i,], bg=col, pch=21, cex=3,add=T)
        text(end_points@coords[i,1], end_points@coords[i,2], labels=i)
    }
}

colors_comunidades = c("#FF004DFF", "#00FFB2FF", "#CCFF00FF", "#0066FFFF")
par(mar=c(.5,.5,.5,.5))

plot_neighborhood_points(centroids, comunidades, colors_comunidades, gpmnetwork, rodovias, lotes)
box()


###################################################
### code chunk number 19: "Saving gpmnetwork in a GPM file"
###################################################
saveGPM(gpmnetwork, "gpmnetwork.gpm", "lotes", "comunidades")


###################################################
### code chunk number 20: "Saving gpmdistance in a GAL file"
###################################################
saveGPM(gpmdistance, "gpmdistance.GAL", "lotes")


###################################################
### code chunk number 21: "Saving gpmdistance in a GWT file"
###################################################
saveGPM(gpmdistance, "gpmdistance.GWT", "lotes", attrib="distance")


